// Classe Principale Parser

public class Parser implements InputListener {
	

	private ParserListener toDispatcher; // Interfaccia verso il Dispatcher
	private InputSocket    fromSocket;   // Classe InputSocket

	   								     // Stato interno Parser	
	private int  Automa_State  = 0;
	private byte parcelType	   = 0;
	private byte LenLSB		   = 0;
	private byte LenMSB		   = 0;
	private int  LenPacket	   = 0;
	private int  LenCount	   = 0;
	private byte infoPackage[] = {0};
	
										 // Costanti di stato Parser e di Errore   
    private final int  ERROR_PARSER_OFFSET = 2000; // Offset errori Parser
    private final int  PARSER_ERROR        = ERROR_PARSER_OFFSET + 1;	
    private final byte STX                 = 0x02;  			
	private final byte ETX                 = 0x03;
	private final int  MAX_PACKET_LEN      = 65535;
	private final int  PARSE_WaitSTX       = 0;
	private final int  PARSE_getLenLSB     = 1;
	private final int  PARSE_getLenMSB     = 2;
	private final int  PARSE_getType       = 3;
	private final int  PARSE_getInfo       = 4;
	private final int  PARSE_getCheck      = 5;
	private final int  PARSE_WaitETX       = 6;
	
   /***************************************
	* Sezione Pubblica (Implementazione)  *
	***************************************/
	
    public Parser (String ipParam, int portParam, ParserListener fromDispatcher) { // Costruttore (String ip_addr, int port_nu)   	    						     
    						     infoPackage  = new byte[MAX_PACKET_LEN];    						     
    						     toDispatcher = fromDispatcher;
                                 fromSocket   = new InputSocket(ipParam, portParam, this); // Costruttore InputSocket
    	                         fromSocket.Open();    	
    	                         reset();
   	                        };     
    	
    public void onError(int i, String s) { // Passo semplicemente verso l'alto l'erore
                                           toDispatcher.onError(i, s);
                                           reset();
    									 };

    public void onChar(byte ch) { // Metodo invocato dall'esterno all'arrivo di un carattere
       
    	switch (Automa_State) { // Selezione stato
		       
		       case PARSE_WaitSTX : { // Attendo STX
										  if (ch == STX) {Automa_State = PARSE_getLenLSB;};
										}; break;
		       
		       case PARSE_getLenLSB : { // Prelevo la lunghezza (LSB)	                                  
		                                  LenLSB = ch;    
		                                  Automa_State = PARSE_getLenMSB;                              		                          
		                                }; break;
		       
		       case PARSE_getLenMSB : { // Prelevo la lunghezza (MSB)
		       							  LenMSB = ch;
		       							  LenPacket = MakeInt(LenLSB, LenMSB);
		       							  LenCount  = LenPacket;
		       							  Automa_State = PARSE_getType;
		                                }; break;
		                                
		                                
			   case PARSE_getType : { // Prelevo il Type del pacchetto
									      parcelType = ch;
									      if (LenPacket == 0) {Automa_State = PARSE_getCheck;}
									                     else {Automa_State = PARSE_getInfo;};
			   	                        }; break;		                                
		       
		       case PARSE_getInfo : { // Tramite la lunghezza costruisco il campo Info
		       						      infoPackage[LenPacket - LenCount] = ch;		       						      
		       						      LenCount = LenCount - 1;	       						      	       						      		       						      
		       						      if (LenCount == 0) {Automa_State = PARSE_getCheck;};
		                                }; break;
		                                
			   case PARSE_getCheck: { // Se le lunghezza  0 allora attendo solo CHECK       						      
		       						      Automa_State = PARSE_WaitETX;			   	
			   					        }; break;
			   	       		       
		       case PARSE_WaitETX : { // Controllo fine pacchetto con ETX
		      				              if (ch == ETX) { toDispatcher.onPacket(parcelType, infoPackage, LenPacket);}
		      						                else { toDispatcher.onError(PARSER_ERROR, "Parser Error");};	      	
		      						      reset();				    
      		      					    }; break;    
		    	
		     }// Fine switch
     
    };  
  
   /***************************************
	* Sezione Privata                     *
	***************************************/
	
   
    private int MakeInt(byte parLenLSB, byte parLenMSB) {
    	                         return ((int)parLenMSB << 8) + (int)parLenLSB;
    	                        };
    	
    private void reset() { // Reset dello stato del Parser
    							 Automa_State = 0;
                                 parcelType   = 0;
	                             LenLSB	      = 0;
	                             LenMSB	      = 0;
                                 LenPacket    = 0;
                                 LenCount	  = 0;
                                };		                      
}